import { RequestOptionsArgs, Response } from '@angular/http';
import { Http } from '@angular/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';

import 'rxjs/add/operator/map';
//import * as $ from 'jquery';

import { Card } from "../models/card";
import { TokenisedCard } from '../models/tokenised-card';
import { Registration } from "../models/registration";
import { CountryMobilePaymentOption } from "../models/country-mobile-payment-option";
import { TransactionToken } from "../models/transaction-token";

declare var require: any;

@Injectable()
export class DirectPayOnline {

    convert = require('xml-js');

    public companyToken: string = "4E003B15-6E43-4682-B46B-B41F6B779C4B";
    //public userToken: string = "DF7D4A21-D8E0-4D8D-B478-E64FC71BAFA1";

    private mobileMoneyOptions: any[] = [];
    private tokenisedCard: TokenisedCard;

    // Live URLs
    private v6_url: string = "https://cors-anywhere.herokuapp.com/https://secure.3gdirectpay.com/API/v6/";
    private v5_url: string = "https://cors-anywhere.herokuapp.com/https://secure.3gdirectpay.com/API/v5/";

    // Test URLs
    //private v6_url: string = "https://cors-anywhere.herokuapp.com/https://secure1.sandbox.directpay.online/API/v6/";
    //private v5_url: string = "https://cors-anywhere.herokuapp.com/https://secure1.sandbox.directpay.online/API/v5/";

    loginRequestStr = "<?xml version='1.0' encoding='utf-8'?> \
  <API3G> \
      <Request>usrAuth</Request> \
      <Username>simon.mswipe</Username> \
      <Password>12345</Password> \
      <CompanyCode>Sales</CompanyCode> \
  </API3G>";

    constructor(public http: Http) {
        console.log('Hello BopoProvider Provider');
    }

    /**
     * 
     */
    public chargeTokenMobile(phoneNumber: string, paymentName: string, country: string, transactionToken: string) {

        let requestStr: string = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
      <CompanyToken>"+ this.companyToken + "</CompanyToken> \
      <Request>ChargeTokenMobile</Request> \
      <TransactionToken>"+ transactionToken + "</TransactionToken> \
      <PhoneNumber>"+ phoneNumber + "</PhoneNumber> \
      <MNO>"+ paymentName + "</MNO> \
      <MNOcountry>" + country + "</MNOcountry> \
    </API3G>"

        console.log("request str" + requestStr);

        this.http.post(
            this.v6_url,
            requestStr
        ).map(res => res.text()).subscribe((res) => {
            //console.log("res: " + res);
            let re = new RegExp('<br>', 'g');
            let res2 = res.replace(re, "");
            var parser = new DOMParser();
            var xmlDoc = parser.parseFromString(res2, "text/xml");
            console.log(xmlDoc);
            let statusCode = xmlDoc.getElementsByTagName("StatusCode")[0].childNodes[0].nodeValue;
            if (statusCode == "130") {
                // success
                return xmlDoc.getElementsByTagName("instructions")[0].childNodes[0].nodeValue;
            } else {
                // error
                return xmlDoc.getElementsByTagName("ResultExplanation")[0].childNodes[0].nodeValue;
            }
            //return "";
        });
    }

    /**
     * getMobilePaymentOptions(): 
     * Gets all registered mobile payment options for this merchant.
     */
    public getMobilePaymentOptionsTS(transactionToken: string) {

        let requestStr = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
      <CompanyToken>"+ this.companyToken + "</CompanyToken> \
      <Request>GetMobilePaymentOptions</Request> \
      <TransactionToken>"+ transactionToken + "</TransactionToken> \
    </API3G>";

        this.http.post(
            this.v5_url,
            requestStr,
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 })).subscribe((res) => {
            //console.log("payOptions: " + payOptions.API3G.paymentoptions.mobileoption[0].country._text);
            var mobileOpts = res.API3G.paymentoptions.mobileoption;
            mobileOpts = this.groupMobilePaymentByCountry(mobileOpts);

            this.mobileMoneyOptions = this.removeDuplicatesBy(country => country.name, mobileOpts);
            console.log(JSON.stringify(this.mobileMoneyOptions));

            return this.mobileMoneyOptions;
        });
    }

    /**
     * 
     */
    private createTransactionToken(amount: string, currency: string, serviceType: string): Observable<any> {

        // First get the serivces for the provider
        var today = new Date();
        var todayStr = today.getFullYear() + '/' + (today.getMonth() + 1) + '/' + today.getDate() + ' ' + today.getHours() + ':' + today.getMinutes();
        console.log("Today str: " + todayStr);

        let requestStr = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
        <CompanyToken>"+ this.companyToken + "</CompanyToken> \
        <Request>createToken</Request> \
        <Transaction> \
            <PaymentAmount>"+ amount + "</PaymentAmount> \
            <PaymentCurrency>"+ currency + "</PaymentCurrency> \
            <CompanyRef>49FKEOA</CompanyRef> \
            <RedirectURL></RedirectURL> \
            <BackURL></BackURL> \
            <CompanyRefUnique>0</CompanyRefUnique> \
            <PTL></PTL> \
        </Transaction> \
        <Services> \
            <Service> \
                <ServiceType>"+ serviceType + "</ServiceType> \
                    <ServiceDescription>Service</ServiceDescription> \
                <ServiceDate>"+ todayStr + "</ServiceDate> \
            </Service> \
        </Services> \
    </API3G>";

        return this.http.post(
            this.v5_url,
            requestStr
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 }));

    }

    private getServicesForProvider(): Observable<any> {
        let servicesReqStr = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
      <Request>getServices</Request> \
      <CompanyToken>"+ this.companyToken + "</CompanyToken> \
      <ProviderToken></ProviderToken> \
    </API3G>";

        return this.http.post(
            this.v5_url,
            servicesReqStr
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 }))
    }

    public createTransToken(amount: string, currency: string) {

        this.getServicesForProvider().subscribe((services) => {
            var serviceID = services.API3G.Services.Service[0].ServiceID._text;
            console.log("ServiceID: " + serviceID);

            this.createTransactionToken(amount, currency, ""+serviceID).subscribe((res) => {
                console.log(JSON.stringify(res));

                return res;
            }); 

        });
    }

    //#region CARD TRANSACTIONS
    public tokeniseCard(card: Card, customerToken: string) {

        let tokeniseCardRequestXML = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
      <CompanyToken>"+ this.companyToken + "</CompanyToken> \
      <Request>addCard</Request> \
      <customerToken>" + customerToken + "</customerToken> \
      <CustomerCreditCard>" + card.cardNumber + "</CustomerCreditCard> \
      <CustomerExpiry>" + card.cardExpMonth + card.cardExpYear + "</CustomerExpiry> \
      <CustomerCvv>" + card.cardCVV + "</CustomerCvv> \
      <CustomerHolderName>" + card.cardHolderName + "</CustomerHolderName> \
    </API3G>";

        console.log(tokeniseCardRequestXML);

        this.http.post(
            this.v6_url,
            tokeniseCardRequestXML
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 })).subscribe((res) => {
            console.log(JSON.stringify(res));

            return res;
        });
    }

    public chargeCard(card: TokenisedCard, transToken: string){

        let requestStr = "<?xml version='1.0' encoding='utf-8'?> \
    <API3G> \
        <CompanyToken>"+ this.companyToken + "</CompanyToken> \
        <Request>chargeTokenRecurrent</Request> \
        <TransactionToken>" + transToken + "</TransactionToken> \
        <subscriptionToken>"+ card.subscriptionToken + "</subscriptionToken> \
    </API3G>";

        console.log("charge card reg str: " + requestStr);

        this.http.post(
            this.v6_url,
            requestStr
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 })).subscribe((res) => {
            console.log(JSON.stringify(res));
      
            return res;
        });
    }

    private deleteCard(card: TokenisedCard, customerToken: string): Observable<any> {
        let requestStr = "<?xml version='1.0' encoding='utf-8'?>" +
            "<API3G>" +
            "<CompanyToken>" + this.companyToken + "</CompanyToken>" +
            "<Request>deleteCard</Request>" +
            "<customerToken>" + customerToken + "</customerToken>" +
            "<subscriptionToken>" + card.subscriptionToken + "</subscriptionToken>" +
            "</API3G>";

        return this.http.post(
            this.v6_url,
            requestStr
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 }));
    }
    //#endregion

    public registerCustomer(registration: Registration) {
        let requestStr = "<API3G>" +
            "<CompanyToken>" + this.companyToken + "</CompanyToken>" +
            "<Request>createAccount</Request>" +
            "<CustomerEmail>" + registration.email + "</CustomerEmail>" +
            "<CustomerFirstName>" + registration.firstName + "</CustomerFirstName>" +
            "<CustomerLastName>" + registration.lastName + "</CustomerLastName>" +
            "<CustomerAddress>" + registration.address + "</CustomerAddress>" +
            "<CustomerCity>" + registration.city + "</CustomerCity>" +
            "<CustomerCountry>" + registration.countryCode + "</CustomerCountry>" +
            "<CustomerDialCode>" + registration.dialCode + "</CustomerDialCode>" +
            "<CustomerPhone>" + registration.phone + "</CustomerPhone>" +
            "<CustomerZip>" + registration.zip + "</CustomerZip>" +
            "<CustomerCreditCard>" + registration.creditCardNo + "</CustomerCreditCard>" +
            "<CustomerExpiry>" + registration.expiryMonth + registration.expiryYear + "</CustomerExpiry>" +
            "<CustomerCvv>" + registration.cvv + "</CustomerCvv>" +
            "<CustomerHolderName>" + registration.cardHolderName + "</CustomerHolderName>" +
            "</API3G>";

        console.log("register request str: " + requestStr);

        this.http.post(
            this.v6_url,
            requestStr
        ).map(res => this.convert.xml2js(res.text(), { compact: true, spaces: 4 })).subscribe((res) => {
            console.log(JSON.stringify(res));

            return res;
        });
    }

    private removeDuplicatesBy(keyFn, array) {
        var mySet = new Set();
        return array.filter(function (x) {
            var key = keyFn(x), isNew = !mySet.has(key);
            if (isNew) mySet.add(key);
            return isNew;
        });
    }

    private groupMobilePaymentByCountry(jsObjects: any[]): CountryMobilePaymentOption[] {

        var opts: CountryMobilePaymentOption[] = [];
        var countryMobilePaymentOption: CountryMobilePaymentOption;
        jsObjects.forEach(function (mobileoption, i) {
            countryMobilePaymentOption = {
                name: mobileoption.country._text,
                countryCode: mobileoption.countryCode._text,
                currency: mobileoption.currency._text,
                cellularPrefix: mobileoption.celluarprefix._text,
                amount: mobileoption.amount._text,
                paymentProviders: [
                    {
                        name: mobileoption.paymentname._text,
                        instructions: mobileoption.instructions._text
                    }
                ]
            };
            jsObjects.forEach(function (mobileoption2, j) {
                if (!Object.is(mobileoption, mobileoption2)) {
                    if (mobileoption.country._text == mobileoption2.country._text) {
                        countryMobilePaymentOption.paymentProviders.push({
                            name: mobileoption2.paymentname._text,
                            instructions: mobileoption2.instructions._text
                        });
                    }
                }
            });
            opts.push(countryMobilePaymentOption);
            //console.log(JSON.stringify(countryMobilePaymentOption));
        });
        return opts;
    }
}